#!/bin/bash

#simulation part of the neural network. This is run by individual chunks in parallel with each other

### 1. Creates directories for each member if they havent already done so, i.e. in the first generation
### 2. Edits the namelists of each member using the corresponding row of the 00.csv parameters file
### 3. Runs the simulation
### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

mode=default

## ALL SHOULD BE THE SAME AS THOSE IN pre.sh
SCRHOME=/gpfs/scratch/bsc32/bsc32786/
MODEL=${SCRHOME}%EXPID%/EXP00/
#OBS=${SCRHOME}pisces1d/ORCA2_OFF_PISCES/obs/Mprof_6901486_L31_month_2015-01-01_2015-12-31.nc
#MODELFILE=PISCES_1m_19600101_19641231_ptrc_T.nc
#OBS=${SCRHOME}pisces1d/ORCA2_OFF_PISCES/obs/Mprof_6901486_L31_5days_2015-01-01_2015-12-31reg.nc
MODELFILE=PISCES_5d_19600101_19641231_ptrc_T.nc
VARIABLES=/gpfs/scratch/bsc32/bsc32786/pisces1d/parameters/variables9.csv
#NEMO=/gpfs/projects/bsc32/models/pisces_off/gapoc/cfgs/mP1D/EXP00/
NEMO=/gpfs/projects/bsc32/models/pisces_off/gapoc/cfgs/mP1D/EXP00/

if [ $mode = 'observed' ]
then
    # path to the observed data. (PATH AND FILE) MUST BE ON THE SAME GRID AS THE MODEL
    OBS=${SCRHOME}pisces1d/obs/Mprof_6901486_L75_5days_2015-01-01_2015-12-31reg.nc
elif [ $mode = 'default' ]
then
    OBS=${SCRHOME}%EXPID%/EXP00/default/$MODELFILE
fi

### 1. Creates directories for each member if they havent already done so, i.e. in the first generation

shopt -s nullglob

chunk=%CHUNK%
printf -v chunk "%04d" $chunk

cd ${MODEL}
if [ ! -d $MODEL$chunk ]; then
    # MAKE EXPID DIRECTORY AND COPY ALL THE CONTENT FROM EXP00 (.XML, EXECUTABLES, AND NAMELISTS)
    mkdir -p $chunk
else
    rm $chunk/*
fi
cd $chunk
cp -R ${NEMO}nemo .
cp -R ${NEMO}namelist* .
cp -R ${NEMO}*.xml .
cp -R ${MODEL}cropped/* .

### 2. Edits the namelists of each member using the corresponding row of the 0000.csv parameters file

NAMELIST=namelist_pisces_cfg

#cd ${MODEL}$chunk/

module load python

function changenls {

nls="$(python - $1 $2 $3 $4 << END
# inputs:
# 1. Parameters file (0000.csv)
# 2. Variables file (variables.csv)
# 3. Namelist (namelist_pisces_ref)
# 4. Chunk number

import os
import pandas as pd
import numpy as np

values=np.genfromtxt('$1')
names=pd.read_csv('$2')

chunk=int('$4')-1

with open('$3','r') as nl:
    with open('temp','w+') as t:
        lines=nl.readlines()
        lines = [i[:-1] for i in lines]
        for l in lines:
            for j in range(len(values[chunk])):
                lstart=l
                lcomm=''
                if '!' in l:
                    lstart=l[:l.find('!')] # so that comments that contain the variable name don't get edited
                    lcomm=l[l.find('!'):]# + '\n'
                if ' ' + names['var'][j]+' ' in lstart and "=" in lstart:
                    lstart = '   '+ names['var'][j] + '  =  ' + str(values[chunk][j]) + ' '
                    break
            t.write(lstart + lcomm + '\n')

os.remove('$3')
os.rename('temp','$3')

END
)"
}

changenls ${MODEL}parameters/0000.csv $VARIABLES ${MODEL}$chunk/$NAMELIST %CHUNK%


### 3. Runs the simulation

cd ${MODEL}$chunk

module purge
module load intel/2017.4
module load impi/2018.4
module load netcdf/4.2
module load hdf5/1.8.19
module load perl

mpirun -np 1 ./nemo

### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

module load netcdf/4.4.1.1
module load mkl
module load python

function getcost {

cost="$(python - $1 $2 $3 $4 $5 << END
# $1 - path to default data $2 - path to observed data $3 - Path to output

mode = '$5'

from netCDF4 import Dataset
import numpy as np
import os
import math

def editmoddata(arr,d,s, mn=None, std=None):
    arr=arr[:,:np.shape(d)[0]]
    #for i in range(s):
    #    arr[i] = np.mean(arr[i::s],0)
    #arr = arr[:s]
    arr[-s:]
    arr[arr == 200000039192547557376] = np.nan
    arr=np.log10(arr)
    if mn:
        #arr = (arr - mn)/std
        return arr
    else:
        mn = np.nanmean(arr)
        std = np.nanstd(arr)
        #arr = (arr - mn)/std
        return arr, mn, std

def getobsdata(A):
    #A = np.array(obs.variables[varname])
    A[A == 0.0] = np.nan
    A=np.log10(A)
    mn = np.nanmean(A)
    std = np.nanstd(A)
    #A = (A - mn)/std
    return A,mn,std

if mode == 'observed':
    obs=Dataset('$2')
    bbp700obs = np.array(obs.variables['bbp700_despiked'])
    bbp700obs=bbp700obs[:,:46]
    bbp700obs, bbp700obsmean, bbp700obsstd = getobsdata(bbp700obs)
    bPOCobs = np.array(obs.variables['bbp700_spike'])
    bPOCobs=bPOCobs[:,:46]
    bPOCobs, bPOCobsmean, bPOCobsstd = getobsdata(bPOCobs)
    chlaobs = np.array(obs.variables['chla_cal'])
    chlaobs=chlaobs[:,:17]
    chlaobs, chlaobsmean, chlaobsstd = getobsdata(chlaobs)
    depths=np.array(obs.variables['deptht'])
    S=np.shape(bbp700obs)[0]
    obs.close()

elif mode == 'default':
    obs=Dataset('$1')

    depths = np.array(obs.variables['deptht'])
    S=int(np.shape(np.array(obs.variables['POC']))[0]/5)
    print('S is',S)
    bbp700obs = (np.array(obs.variables['POC'])[-S:, :, 1, 1]+np.array(obs.variables['PHY'])[-S:, :, 1, 1]+np.array(obs.variables['ZOO'])[-S:, :, 1, 1])*0.001
    bPOCobs = (np.array(obs.variables['GOC'])[-S:, :, 1, 1]+np.array(obs.variables['PHY2'])[-S:, :, 1, 1])*0.001
    chlaobs = (np.array(obs.variables['NCHL'])[-S:, :, 1, 1]+np.array(obs.variables['DCHL'])[-S:, :, 1, 1])
    bbp700obs=bbp700obs[:,:46]
    bPOCobs=bPOCobs[:,:46]
    chlaobs=chlaobs[:,:17]
    bbp700obs, bbp700obsmean, bbp700obsstd = editmoddata(bbp700obs,depths,S)
    bPOCobs, bPOCobsmean, bPOCobsstd = editmoddata(bPOCobs,depths,S)
    chlaobs, chlaobsmean, chlaobsstd = editmoddata(chlaobs,depths,S)

# get the model data
os.chdir('$3')
mod = Dataset('$4')
bbp700mod = (np.array(mod.variables['POC'])[-S:, :, 1, 1]+np.array(mod.variables['PHY'])[-S:, :, 1, 1]+np.array(mod.variables['ZOO'])[-S:, :, 1, 1])*0.001
bPOCmod = (np.array(mod.variables['GOC'])[-S:, :, 1, 1]+np.array(mod.variables['PHY2'])[-S:, :, 1, 1])*0.001
chlamod = np.array(mod.variables['NCHL'])[-S:, :, 1, 1]
bbp700mod=bbp700mod[:,:46]
bPOCmod=bPOCmod[:,:46]
chlamod=chlamod[:,:17]
#mod.close()

bbp700mod=editmoddata(bbp700mod,depths,S, mn=bbp700obsmean, std=bbp700obsstd)
bPOCmod=editmoddata(bPOCmod,depths,S, mn=bPOCobsmean, std=bPOCobsstd)
chlamod=editmoddata(chlamod,depths,S, mn=chlaobsmean, std=chlaobsstd)

if mode == 'observed':
    bbp700mod = bbp700mod[:,:np.shape(bbp700obs)[1]]
    bPOCmod = bPOCmod[:,:np.shape(bPOCobs)[1]]
    chlamod = chlamod[:,:np.shape(chlaobs)[1]]

f = 'dyna_grid_T.nc'
dynagrid=Dataset(f)
mldkz5 = np.array(dynagrid.variables['mldkz5'])[:,1,1]
mldkz5 = np.array([sum(mldkz5[5*i:5*i+5])/5 for i in range(int(len(mldkz5)/5))])

for i in range(len(mldkz5)):

    ds = np.where(depths<mldkz5[i])
    bbp700mod[i, ds] = np.nan
    bbp700obs[i, ds] = np.nan

pardm=np.array(mod.variables['PARDM'])[-S:,:46,1,1]
#pardm=pardm[-S:,:46,1,1]
bbp700mod[pardm > 0.4] = np.nan
bbp700obs[pardm > 0.4] = np.nan
mod.close()

# calculating the statistical parameters

names=['bias','normstd','R','RMSE']#,'cost']

bias=np.nanmean(bbp700mod-bbp700obs)
#mbias=bias/bbp700refbias
mstd=np.nanstd(bbp700mod)/np.nanstd(bbp700obs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(bbp700obs.flatten()), np.ma.masked_invalid(bbp700mod.flatten()))[0,1])
S3=1.-(((1.+R)**4.)/(4.*(mstd**2+mstd**(-2)+2))) #S2
RMSE=math.sqrt(np.nansum((bbp700obs-bbp700mod)**2)/bbp700obs.size)
#ST=math.sqrt((mbias**2)+(S3**2))

stats=[bias,mstd,R,RMSE]#,ST]
for i in range(len(names)):
    np.savetxt('bbp700'+names[i]+'.csv',[stats[i]])

bias=np.nanmean(bPOCmod-bPOCobs)
#mbias=bias/bPOCrefbias
mstd=np.nanstd(bPOCmod)#/np.nanstd(bPOCobs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(bPOCobs.flatten()), np.ma.masked_invalid(bPOCmod.flatten()))[0,1])
S3=1.-(((1.+R)**4.)/(4.*(mstd**2+mstd**(-2)+2))) #S2
RMSE=math.sqrt(np.nansum((bPOCobs-bPOCmod)**2)/bPOCobs.size)
#ST=math.sqrt((mbias**2)+(S3**2))

stats=[bias,mstd,R,RMSE]#,ST]
for i in range(len(names)):
    np.savetxt('bPOC'+names[i]+'.csv',[stats[i]])

bias=np.nanmean(chlamod-chlaobs)
#mbias=bias/chlarefbias
mstd=np.nanstd(chlamod)#/np.nanstd(chlaobs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(chlaobs.flatten()), np.ma.masked_invalid(chlamod.flatten()))[0,1])
S3=1.-(math.exp(-((mstd-1.)**2)/0.18))*((1+R)/2)
RMSE=math.sqrt(np.nansum((chlaobs-chlamod)**2)/chlaobs.size)
#ST=math.sqrt((mbias**2)+(S3**2))

stats=[bias,mstd,R,RMSE]#,ST]
for i in range(len(names)):
    np.savetxt('chla'+names[i]+'.csv',[stats[i]])

END
)"
}

getcost ${MODEL}default/$MODELFILE $OBS $MODEL$chunk $MODELFILE $mode


